@@ -1,37 +1,41 @@
 /****************************************************************************
 **
-** Copyright (C) 2008 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
 ** Contact: Qt Software Information (qt-info@nokia.com)
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial Usage
 ** Licensees holding valid Qt Commercial licenses may use this file in
 ** accordance with the Qt Commercial License Agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and Nokia.
 **
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+**
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License versions 2.0 or 3.0 as published by the Free
-** Software Foundation and appearing in the file LICENSE.GPL included in
-** the packaging of this file.  Please review the following information
-** to ensure GNU General Public Licensing requirements will be met:
-** http://www.fsf.org/licensing/licenses/info/GPLv2.html and
-** http://www.gnu.org/copyleft/gpl.html.  In addition, as a special
-** exception, Nokia gives you certain additional rights. These rights
-** are described in the Nokia Qt GPL Exception version 1.3, included in
-** the file GPL_EXCEPTION.txt in this package.
-**
-** Qt for Windows(R) Licensees
-** As a special exception, Nokia, as the sole copyright holder for Qt
-** Designer, grants users of the Qt/Eclipse Integration plug-in the
-** right for the Qt/Eclipse Integration to link to functionality
-** provided by Qt Designer and its related libraries.
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
 ** If you are unsure which license is appropriate for your use, please
 ** contact the sales department at qt-sales@nokia.com.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -64,15 +68,18 @@ QT_BEGIN_NAMESPACE
 QMultiInputContext::QMultiInputContext()
     : QInputContext(), current(-1)
 {
-    QStringList keys = QInputContextFactory::keys();
+    keys = QInputContextFactory::keys();
     for (int i = keys.size()-1; i >= 0; --i)
         if (keys.at(i).contains(QLatin1String("imsw")))
             keys.removeAt(i);
 
-    QString def = QLatin1String(getenv("QT_IM_MODULE"));
+    QString def = QLatin1String(getenv("QT4_IM_MODULE"));
+    if (def.isEmpty())
+        def = QLatin1String(getenv("QT_IM_MODULE"));
     if (def.isEmpty()) {
-	QSettings settings;
-        def = settings.value(QLatin1String("/qt/DefaultInputMethod"), QLatin1String("xim")).toString();
+        QSettings settings(QSettings::UserScope, QLatin1String("Trolltech"));
+        settings.beginGroup(QLatin1String("Qt"));
+        def = settings.value(QLatin1String("DefaultInputMethod"), QLatin1String("xim")).toString();
     }
     current = keys.indexOf(def);
     if (current < 0)
@@ -84,20 +91,23 @@ QMultiInputContext::QMultiInputContext()
 
     QActionGroup *group = new QActionGroup(this);
     for (int i = 0; i < keys.size(); ++i) {
-        slaves.append(QInputContextFactory::create(keys.at(i), this));
-        QAction *a = menu->addAction(slaves.at(i)->identifierName());
+        slaves.append(0);
+        const QString key = keys.at(i);
+        QAction *a = menu->addAction(QInputContextFactory::displayName(key));
+        a->setData(key);
         a->setCheckable(true);
         group->addAction(a);
-        if (i == current)
+        if (i == current) {
+            slaves.replace(current, QInputContextFactory::create(key, this));
             a->setChecked(true);
+        }
     }
     connect(group, SIGNAL(triggered(QAction*)), this, SLOT(changeSlave(QAction*)));
-
-
 }
 
 QMultiInputContext::~QMultiInputContext()
 {
+    delete menu;
 }
 
 
@@ -182,7 +192,9 @@ QList<QAction *> QMultiInputContext::actions()
 void QMultiInputContext::changeSlave(QAction *a)
 {
     for (int i = 0; i < slaves.size(); ++i) {
-        if (slaves.at(i)->identifierName() == a->text()) {
+        if (keys.at(i) == a->data().toString()) {
+            if (slaves.at(i) == 0)
+                slaves.replace(i, QInputContextFactory::create(keys.at(i), this));
             current = i;
             return;
         }
