@@ -1,46 +1,45 @@
 /****************************************************************************
 **
-** Copyright (C) 2007-2008 Trolltech ASA. All rights reserved.
+** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Contact: Qt Software Information (qt-info@nokia.com)
 **
 ** This file is part of the Qt Designer of the Qt Toolkit.
 **
-** This file may be used under the terms of the GNU General Public
-** License versions 2.0 or 3.0 as published by the Free Software
-** Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file.  Alternatively you may (at
-** your option) use any later version of the GNU General Public
-** License if such license has been publicly approved by Trolltech ASA
-** (or its successors, if any) and the KDE Free Qt Foundation. In
-** addition, as a special exception, Trolltech gives you certain
-** additional rights. These rights are described in the Trolltech GPL
-** Exception version 1.2, which can be found at
-** http://www.trolltech.com/products/qt/gplexception/ and in the file
-** GPL_EXCEPTION.txt in this package.
+** $QT_BEGIN_LICENSE:LGPL$
+** Commercial Usage
+** Licensees holding valid Qt Commercial licenses may use this file in
+** accordance with the Qt Commercial License Agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Nokia.
 **
-** Please review the following information to ensure GNU General
-** Public Licensing requirements will be met:
-** http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-** you are unsure which license is appropriate for your use, please
-** review the following information:
-** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-** or contact the sales department at sales@trolltech.com.
+** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Trolltech, as the sole
-** copyright holder for Qt Designer, grants users of the Qt/Eclipse
-** Integration plug-in the right for the Qt/Eclipse Integration to
-** link to functionality provided by Qt Designer and its related
-** libraries.
+** In addition, as a special exception, Nokia gives you certain
+** additional rights. These rights are described in the Nokia Qt LGPL
+** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
+** package.
 **
-** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-** A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-** granted herein.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+** If you are unsure which license is appropriate for your use, please
+** contact the sales department at qt-sales@nokia.com.
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
+#include "abstractsettings_p.h"
 #include "qtresourceview_p.h"
 #include "qtresourcemodel_p.h"
 #include "qtresourceeditordialog_p.h"
@@ -68,7 +67,6 @@
 #include <QtGui/QMenu>
 #include <QtGui/QDrag>
 #include <QtCore/QMimeData>
-#include <QtCore/QSettings>
 #include <QtXml/QDomDocument>
 
 QT_BEGIN_NAMESPACE
@@ -153,6 +151,7 @@ public:
     void applyExpansionState();
     void restoreSettings();
     void saveSettings();
+    void updateActions();
 
     QPixmap makeThumbnail(const QPixmap &pix) const;
 
@@ -175,6 +174,7 @@ public:
 
     bool m_ignoreGuiSignals;
     QString m_settingsKey;
+    bool m_resourceEditingEnabled;
 };
 
 QtResourceViewPrivate::QtResourceViewPrivate(QDesignerFormEditorInterface *core) :
@@ -187,7 +187,8 @@ QtResourceViewPrivate::QtResourceViewPrivate(QDesignerFormEditorInterface *core)
     m_editResourcesAction(0),
     m_reloadResourcesAction(0),
     m_copyResourcePathAction(0),
-    m_ignoreGuiSignals(false)
+    m_ignoreGuiSignals(false),
+    m_resourceEditingEnabled(true)
 {
 }
 
@@ -196,11 +197,11 @@ void QtResourceViewPrivate::restoreSettings()
     if (m_settingsKey.isEmpty())
         return;
 
-    QSettings settings;
-    settings.beginGroup(m_settingsKey);
+    QDesignerSettingsInterface *settings = m_core->settingsManager();
+    settings->beginGroup(m_settingsKey);
 
-    m_splitter->restoreState(settings.value(QLatin1String(SplitterPosition)).toByteArray());
-    settings.endGroup();
+    m_splitter->restoreState(settings->value(QLatin1String(SplitterPosition)).toByteArray());
+    settings->endGroup();
 }
 
 void QtResourceViewPrivate::saveSettings()
@@ -208,16 +209,18 @@ void QtResourceViewPrivate::saveSettings()
     if (m_settingsKey.isEmpty())
         return;
 
-    QSettings settings;
-    settings.beginGroup(m_settingsKey);
+    QDesignerSettingsInterface *settings = m_core->settingsManager();
+    settings->beginGroup(m_settingsKey);
 
-    settings.setValue(QLatin1String(SplitterPosition), m_splitter->saveState());
-    settings.endGroup();
+    settings->setValue(QLatin1String(SplitterPosition), m_splitter->saveState());
+    settings->endGroup();
 }
 
 void QtResourceViewPrivate::slotEditResources()
 {
-    const QString selectedResource = QtResourceEditorDialog::editResources(m_resourceModel, m_core->dialogGui(), q_ptr);
+    const QString selectedResource
+            = QtResourceEditorDialog::editResources(m_core, m_resourceModel,
+                                                    m_core->dialogGui(), q_ptr);
     if (!selectedResource.isEmpty())
         q_ptr->selectResource(selectedResource);
 }
@@ -281,11 +284,22 @@ QPixmap QtResourceViewPrivate::makeThumbnail(const QPixmap &pix) const
     return QPixmap::fromImage(img);
 }
 
-void QtResourceViewPrivate::slotResourceSetActivated(QtResourceSet *resourceSet)
+void QtResourceViewPrivate::updateActions()
 {
-    const bool resourceActive = resourceSet;
+    bool resourceActive = false;
+    if (m_resourceModel)
+        resourceActive = m_resourceModel->currentResourceSet();
+
+    m_editResourcesAction->setVisible(m_resourceEditingEnabled);
     m_editResourcesAction->setEnabled(resourceActive);
     m_reloadResourcesAction->setEnabled(resourceActive);
+}
+
+void QtResourceViewPrivate::slotResourceSetActivated(QtResourceSet *resourceSet)
+{
+    Q_UNUSED(resourceSet)
+
+    updateActions();
 
     storeExpansionState();
     const QString currentPath = m_itemToPath.value(m_treeWidget->currentItem());
@@ -445,7 +459,7 @@ QtResourceView::QtResourceView(QDesignerFormEditorInterface *core, QWidget *pare
     connect(d_ptr->m_reloadResourcesAction, SIGNAL(triggered()), this, SLOT(slotReloadResources()));
     d_ptr->m_reloadResourcesAction->setEnabled(false);
 
-    d_ptr->m_copyResourcePathAction = new QAction(tr("Copy Path"), this);
+    d_ptr->m_copyResourcePathAction = new QAction(qdesigner_internal::createIconSet(QLatin1String("editcopy.png")), tr("Copy Path"), this);
     connect(d_ptr->m_copyResourcePathAction, SIGNAL(triggered()), this, SLOT(slotCopyResourcePath()));
     d_ptr->m_copyResourcePathAction->setEnabled(false);
 
@@ -576,6 +590,17 @@ void QtResourceView::setResourceModel(QtResourceModel *model)
     d_ptr->slotResourceSetActivated(d_ptr->m_resourceModel->currentResourceSet());
 }
 
+bool QtResourceView::isResourceEditingEnabled() const
+{
+    return d_ptr->m_resourceEditingEnabled;
+}
+
+void QtResourceView::setResourceEditingEnabled(bool enable)
+{
+    d_ptr->m_resourceEditingEnabled = enable;
+    d_ptr->updateActions();
+}
+
 void QtResourceView::setDragEnabled(bool dragEnabled)
 {
     d_ptr->m_listWidget->setDragEnabled(dragEnabled);
@@ -662,12 +687,14 @@ public:
     void slotResourceSelected(const QString &resource) { setOkButtonEnabled(!resource.isEmpty()); }
     void setOkButtonEnabled(bool v)                    { m_box->button(QDialogButtonBox::Ok)->setEnabled(v); }
 
+    QDesignerFormEditorInterface *m_core;
     QtResourceView *m_view;
     QDialogButtonBox *m_box;
 };
 
 QtResourceViewDialogPrivate::QtResourceViewDialogPrivate(QDesignerFormEditorInterface *core) :
     q_ptr(0),
+    m_core(core),
     m_view(new QtResourceView(core)),
     m_box(new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel))
 {
@@ -692,23 +719,23 @@ QtResourceViewDialog::QtResourceViewDialog(QDesignerFormEditorInterface *core, Q
     d_ptr->setOkButtonEnabled(false);
     d_ptr->m_view->setResourceModel(core->resourceModel());
 
-    QSettings settings;
-    settings.beginGroup(QLatin1String(ResourceViewDialogC));
+    QDesignerSettingsInterface *settings = core->settingsManager();
+    settings->beginGroup(QLatin1String(ResourceViewDialogC));
 
-    if (settings.contains(QLatin1String(Geometry)))
-        setGeometry(settings.value(QLatin1String(Geometry)).toRect());
+    if (settings->contains(QLatin1String(Geometry)))
+        setGeometry(settings->value(QLatin1String(Geometry)).toRect());
 
-    settings.endGroup();
+    settings->endGroup();
 }
 
 QtResourceViewDialog::~QtResourceViewDialog()
 {
-    QSettings settings;
-    settings.beginGroup(QLatin1String(ResourceViewDialogC));
+    QDesignerSettingsInterface *settings = d_ptr->m_core->settingsManager();
+    settings->beginGroup(QLatin1String(ResourceViewDialogC));
 
-    settings.setValue(QLatin1String(Geometry), geometry());
+    settings->setValue(QLatin1String(Geometry), geometry());
 
-    settings.endGroup();
+    settings->endGroup();
 
     delete d_ptr;
 }
@@ -723,6 +750,16 @@ void QtResourceViewDialog::selectResource(const QString &path)
     d_ptr->m_view->selectResource(path);
 }
 
+bool QtResourceViewDialog::isResourceEditingEnabled() const
+{
+    return d_ptr->m_view->isResourceEditingEnabled();
+}
+
+void QtResourceViewDialog::setResourceEditingEnabled(bool enable)
+{
+    d_ptr->m_view->setResourceEditingEnabled(enable);
+}
+
 QT_END_NAMESPACE
 
 #include "moc_qtresourceview_p.cpp"
